/* public/css/ado/admin.css */
@charset "UTF-8";

.footer,
.main.container {
  padding: 1ex;
  margin: 0;
  padding-top: 2.5em;
  height: 97vh;
  max-width: 100%;
  overflow: auto;
  background: none;
  border: none;
  box-shadow: none;
}

#admin_menu {
  border-radius: 5px !important;
}

.main .top.attached.pushable > .pusher {
  padding-left:1ex;
  padding-right: 8.4em;/* otherwise the tab segment right border is not viible*/
}

.main .top.attached.pushable > .pusher .tab.segment {
  z-index: inherit;
}
#tab_body #users_list.ui.grid,
#tab_body #users_list.ui.grid > .row,
#tab_body #users_list.ui.grid > .row > .column {
  padding-top: 0;
  padding-bottom: 0;
  padding-left: 1ex;
  padding-right: 1ex;
  margin: 0;
}
#tab_body .ui.divider {
  margin: 0.5ex;
}
#tab_body #users_list.grid {
  height: 65vh;
  overflow: auto;
}
